Public Class frmWKTest
  Public WithEvents serialPort As New IO.Ports.SerialPort    'Note "WITH EVENTS"

  ' See DevX forum at http://www.devx.com/dotnet/Article/31001 for a good article on serial port.
  'Many of the routines here could be compressed into smaller or common code but in the interest
  'of allowing cut and paste extracts of selected parts, simple duplication is employed.

  Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
    'Load up the available ports to choose.
    For i As Integer = 0 To _
       My.Computer.Ports.SerialPortNames.Count - 1
      cbbCOMPorts.Items.Add( _
         My.Computer.Ports.SerialPortNames(i))
    Next
    'Set some startup items and properties
    btnDisconnect.Enabled = False
    Call SetWinkeyGlobals()
  End Sub

  Private Sub DataReceived(ByVal sender As Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) _
     Handles serialPort.DataReceived    'Event handler for serial receive
    txtDataReceived.Invoke(New myDelegate(AddressOf updateTextBox), New Object() {})
  End Sub

  Public Delegate Sub myDelegate()
  Public Sub updateTextBox()   'Data received on each event is passed here on the main thread.
    Dim Ret As Boolean = False
    Dim RXChar As String = ""
    Dim RXInt As Integer
    Dim RXStatus As Byte = 0
    Dim RXBuffCount As Integer = 0
    Dim WPMSpeed As Integer = 0
    'All bytes returned from WinKey to the host should be processed to determine if they are 
    '1 Status 3 MSBs =110
    '2 SpeedPot or PB Change (WK2 Mode Only) 2 MSBs = 10
    '3 A response to an information request from the host to WK
    '4 or echoback characters when echo is on.  
    '      MSB =0 if Echo, If Paddle Brk is on, it is a paddle echo, else a serial echo
    RXInt = serialPort.ReadByte
    Me.txtCmdReturn.Text = Me.txtCmdReturn.Text + Str(RXInt) + " "
    Me.txtCmdReturn.ScrollToCaret()   'Position txt box to bottom of the text string.

    'These could be nested and Hex used....
    If RXInt >= 20 Then  'Checking for Version on Open
      If RXInt <= 31 Then 'Looking for 20 or more....
        RXChar = " Version " + Str(RXInt) + " " + vbCrLf
      End If
    End If

    If RXInt >= 32 Then  'Checking for ASCII Chars on Echo back.
      If RXInt <= 127 Then
        RXChar = Convert.ToChar(RXInt)
        Me.txtEchoedChars.Text = Me.txtEchoedChars.Text + RXChar + " "
        Me.txtEchoedChars.ScrollToCaret()   'Position txt box to bottom of the text string.
      End If
    End If

    If RXInt >= 128 Then  'Checking for Speed Pot returns
      If RXInt <= 191 Then 'Status bytes have "10" as 2 MSBits.
        RXChar = vbCrLf + "Speed Pot " + Str(RXInt - 128) + " " 'Remove the top 2 bits, 6 bits left are speed - MinPot
      End If
    End If

    If RXInt >= 192 Then  'Checking for Status Byte returns
      If RXInt <= 223 Then 'Status bytes have "110" as 3 MSBits.
        RXChar = vbCrLf + "Status byte "
        Ret = WK2Status(RXInt) 'remove top 3 bits before call
      End If
    End If

    With txtDataReceived
      .Font = New Font("Garamond", 12.0!, FontStyle.Bold)
      .SelectionColor = Color.Red
      .AppendText(RXChar)
      .ScrollToCaret()
    End With
    txtDataToSend.Focus()
  End Sub

  Private Sub btnSend_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnSend.Click
    'Enter or button will send text in the text box...
    Dim SendChar As String
    SendChar = txtDataToSend.Text
    Try
      serialPort.Write(UCase(SendChar))  'WinKey likes uppercase chars....Only
      'With txtDataReceived
      '  .SelectionColor = Color.Black
      '  .AppendText("-> " + txtDataToSend.Text & vbCrLf)  '& vbCrLf
      '  .ScrollToCaret()
      'End With
      txtDataToSend.Text = "" 'String.Empty
    Catch ex As Exception
      MsgBox(ex.ToString)
    End Try
    txtDataToSend.Focus()
  End Sub

  Private Sub cmdSendChars_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdSendChars.Click
    Call SendCWChar()
  End Sub

  Public Sub SendCWChar()   'This is the simple send a K button....
    Dim SendChar As String
    SendChar = "K"
    Try
      serialPort.Write(UCase(SendChar))  'WinKey likes uppercase chars....Only
    Catch ex As Exception
      MsgBox(ex.ToString)
    End Try
    txtDataToSend.Focus()
  End Sub

  Private Sub btnConnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnConnect.Click
    Call ConnectPort()
  End Sub

  Private Sub btnDisconnect_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDisconnect.Click
    Call DisconnectPort()
  End Sub

  Private Sub cmdOpenHost_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdOpenHost.Click
    Call HostOpen()
  End Sub

  Private Sub cmdHostClose_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdHostClose.Click
    Call HostClose()
  End Sub

  Private Sub cmdGetPot_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetPot.Click
    Call GetSpeedPot()
  End Sub

  Private Sub cmdClearWKBuffer_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdClearWKBuffer.Click
    Call ClearWKBuffer()
  End Sub

  Private Sub cmdPause_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPause.Click
    Call PauseResume()
  End Sub

  Private Sub cmdTune_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdTune.Click
    Call WKTune()
  End Sub

  Private Sub cbxTone_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxTone.SelectedIndexChanged
    Call ToneAndPaddle()
  End Sub

  Private Sub cbxPaddleOnly_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxPaddleOnly.CheckedChanged
    Call ToneAndPaddle()
  End Sub

  Private Sub cbxWPM_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cbxWPM.SelectedIndexChanged
    Call SetWPM()
  End Sub

  Private Sub cmdGetDefaults_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdGetDefaults.Click
    Call RequestWK2Status()

  End Sub

  Private Sub tmrMessage_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrMessage.Tick

    Me.tmrMessage.Stop()

    'lblCWBusy.Text = ""
    'lblCWBusy.BackColor = Color.White

  End Sub

  Private Sub cmdEchoToggle_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdEchoToggle.Click
    Call EchoToggle()

  End Sub
End Class
